from lxml import html as htm
from html import escape
from pandas import DataFrame
from pandas import to_datetime
from pandas import read_excel
from numpy import where
import cssselect
import requests as rq
import locale
import platform
import regex as re

root_url = 'https://www.maxqda.de/support/forum/viewforum.php?f=23'

df_base_pages = DataFrame()
df_base_pages['base_page_url'] = [root_url + '&start=' + str(x) for x in range(0, 400, 50)]
df_base_pages['html'] = [rq.get(x).text for x in df_base_pages.base_page_url]
print(df_base_pages)

base_url = 'https://www.maxqda.de/support/forum'

df_threads = DataFrame()
for index, row in df_base_pages.iterrows():
    print(row.base_page_url)
    doc = htm.document_fromstring(row.html)
    df_tmp = DataFrame()
    df_tmp['thread_url'] = [base_url + e.get('href')[1:] for e in doc.cssselect('li.row dt a.topictitle')]
    df_tmp['title'] = [e.text for e in doc.cssselect('li.row dt a.topictitle')]
    df_tmp['posts'] = [e.text for e in doc.cssselect('li.row dd.posts')]
    df_tmp['views'] = [e.text for e in doc.cssselect('li.row dd.views')]
    df_tmp['base_page_url'] = row.base_page_url
    df_tmp['html'] = [rq.get(x).text for x in df_tmp.thread_url]
    df_threads = df_threads.append(df_tmp)
print(df_threads.head())

df_postings = DataFrame()
for index, row in df_threads.iterrows():
    # print(row.thread_url)
    thread = htm.document_fromstring(row.html)
    postings = [e for e in thread.cssselect('div.post')]
    for cp in postings:
        for br in cp.cssselect('br'):
            br.tail = "\n" + br.tail if br.tail else "\n"
    df_tmp = DataFrame()
    df_tmp['header'] = [cp.cssselect('h2')[0].text_content().strip() for cp in postings]
    df_tmp['datum'] = [cp.cssselect('p.author')[0].text_content().strip() for cp in postings]
    df_tmp['autor'] = [cp.cssselect('dl.postprofile dt')[0].text_content().strip() for cp in postings]
    df_tmp['content'] = [cp.cssselect('div.content')[0].text_content().strip() for cp in postings]
    df_tmp['first_in_thread'] = [(cp.cssselect('div.postbody h2')[0]).get('class') == 'first' for cp in postings]
    df_tmp['base_page_url'] = row.base_page_url
    df_tmp['thread_url'] = row.thread_url
    df_tmp['title'] = row.title.strip()
    df_tmp['posts'] = int(row.posts)
    df_tmp['views'] = int(row.views)
    df_postings = df_postings.append(df_tmp)

# print(df_postings.head())
# df_postings.to_excel("postings_save.xlsx")
# df_postings = read_excel("postings_save.xlsx")

if platform.system() == "Windows":
    locale.setlocale(locale.LC_ALL, 'German_Germany.1252')
    df_postings.datum = df_postings.datum.str.replace('Mär', 'Mrz')
else:
    locale.setlocale(locale.LC_ALL, 'de_DE')

df_postings['datum_zeit'] = to_datetime(df_postings.datum, format="%d %b %Y, %H:%M")
df_postings['post_datum'] = df_postings.datum_zeit.dt.strftime("%Y-%m-%d")
df_postings['uhrzeit'] = df_postings.datum_zeit.dt.strftime("%H:%M:%S")
df_postings['time_stamp'] = df_postings.datum_zeit.dt.strftime("%Y-%m-%d %H:%M:%S")
df_postings['jahr'] = df_postings.datum_zeit.dt.strftime("%Y")
df_postings['monat'] = df_postings.datum_zeit.dt.strftime("%m %B")
df_postings['jahr_monat'] = df_postings.jahr + "-" + df_postings.monat
df_postings['tag'] = df_postings.datum_zeit.dt.strftime("%d")
df_postings['wtag'] = df_postings.datum_zeit.dt.strftime("%u %A")
df_postings['stunde'] = df_postings.datum_zeit.dt.strftime("%H")
df_postings['size'] = df_postings.content.str.len()
df_postings['maxqda_ver'] = df_postings.content.str.extract(r"(?<=^Version: )(.*$)", re.MULTILINE, expand=False)
df_postings['os_ver'] = df_postings.content.str.extract(r"(?<=^System: )(.*$)", re.MULTILINE, expand=False)

df_postings['thread_datum'] = df_postings.groupby('thread_url').post_datum.transform('first')
df_postings['thread_zeit'] = df_postings.groupby('thread_url').datum_zeit.transform('first')
df_postings['first_poster'] = df_postings.groupby('thread_url').autor.transform('first')

df_postings['typ'] = where(df_postings.autor == df_postings.first_poster, "Frage", "Antwort")
df_postings['Dokumentgruppe'] = (df_postings.thread_zeit.dt.strftime("%Y-%m\\%d") + "-" + df_postings.title).str[:63]
df_postings['Dokumentname'] = df_postings.time_stamp + "_" + df_postings.autor

# df_postings.to_excel("postings_modified.xlsx")

df_postings[['Dokumentgruppe', 'Dokumentname', 'typ', 'first_in_thread', 'os_ver', 'maxqda_ver']].head(10)

df_postings_maxqda = df_postings.copy()
df_postings_maxqda = \
    df_postings_maxqda.reindex(columns=
                               ['Dokumentgruppe',
                                'Dokumentname',
                                'base_page_url',
                                'thread_datum', 'thread_zeit',
                                'posts', 'views', 'title',
                                'post_datum', 'time_stamp',
                                'tag', 'wtag', 'stunde', 'size',
                                'autor', 'header', 'content', 'typ',
                                'maxqda_ver', 'os_ver',
                                'first_in_thread', 'first_poster'
                                ]).copy()

df_postings_maxqda.to_excel("postings_maxqda.xlsx")
df_postings = df_postings.fillna('n.a.')

cat_var = ['thread_datum', 'post_datum', 'jahr', 'monat', 'jahr_monat', 'tag', 'wtag', 'stunde',
           'maxqda_ver', 'os_ver', 'typ']

for col in cat_var:
    df_postings.loc[:, col] = col + "\\" + df_postings.loc[:, col]

df_postings.content = df_postings.content.str.replace("\n", "<br>")
df_postings.content = escape(df_postings.content)

postings_grouped = df_postings.groupby('Dokumentgruppe')

html_file = open("threads_maxqda.html", "w", encoding="UTF-8")

html_file.write("""<!DOCTYPE html>
<html lang="de">
<head>
<meta charset="UTF-8"/>
</head>
<body>
""")

for Dokumentgruppe, df_group in postings_grouped:
    html_file.write("#TEXT " + Dokumentgruppe + "<br>\n")
    for index, row in df_group.iterrows():
        html_file.write("#CODE ")
        html_file.write(
            row.thread_datum + "&&" +
            row.post_datum + "&&" +
            row.jahr + "&&" +
            row.monat + "&&" +
            row.jahr_monat + "&&" +
            row.tag + "&&" +
            row.wtag + "&&" +
            row.stunde + "&&" +
            row.maxqda_ver + "&&" +
            row.os_ver + "&&" +
            row.typ
        )
        html_file.write("<br>\n<strong>")
        html_file.write(row.time_stamp + "<br>\n")
        html_file.write(row.autor + "<br>\n")
        html_file.write("</strong><br>\n")
        html_file.write(row.content)
        html_file.write("<br>\n#ENDCODE")
        html_file.write("<br><br>\n\n")

html_file.write("""    
</body>
</html>
""")

html_file.close()

df_thread_numeric_vars = (df_postings[["Dokumentgruppe", "views", "posts", "size"]]
                          .groupby("Dokumentgruppe")
                          .agg({"views": "first", "posts": "first", "size": "sum"}))

df_thread_numeric_vars['Dokumentname'] = df_thread_numeric_vars.index
df_thread_numeric_vars["Dokumentgruppe"] = "Forumthreads"
df_thread_numeric_vars = df_thread_numeric_vars.reindex(
    columns=['Dokumentgruppe', 'Dokumentname', 'posts', 'views', 'size'])

df_thread_numeric_vars.to_excel("threads_numeric_maxqda.xlsx", index=False)

df_thread_numeric_vars.info()
